#!/usr/bin/perl

=pod 

=head1 NAME

convert_rfam_gff3.pl

=head1 SYNOPSIS

convert_rfam_gff3.pl --infile rfam.gff --outfile rfam.gff3 [--add_parent gene]

=head1 DESCRIPTION

Convert rfam_scan-1.0.2.pl output in GFF3 format (http://www.sequenceontology.org/gff3.shtml)

=cut

BEGIN
{
	my ($dir,$file) = $0 =~ /(.+)\/(.+)/;
	unshift (@INC,"$dir/../../corelib");
	unshift (@INC,"$dir/../../lib");
}

use strict; 
use warnings;
use Carp;

use ParamParser;
use Logger;
use General;

MAIN:
{
	my $o_param = New ParamParser('GETOPTLONG', "infile=s", "outfile=s", "add_parent=s");

	$o_param->SetUsage(my $usage=sub { &Usage(); } );
	$o_param->AssertFileExists('infile');
	$o_param->AssertDefined('outfile');

	my $infile  = $o_param->Get('infile');
	my $outfile = $o_param->Get('outfile');
	my $f_in    = &General::GetStreamIn($infile);
	my $f_out   = &General::GetStreamOut($outfile);

	# Skip the first three lines
	# Print gff3 header
	my $cpt=1;
	print $f_out "##gff-version 3\n";
	while ( my $line = <$f_in> )
	{
		chomp($line);
        next if ( $line =~ /^#/ || $line eq '' );
		#Rm2011G.c	Rfam	similarity	1146929	1147015	53.81	+	.	evalue=5.01e-12;gc-content=60;id=tRNA.1;model_end=71;model_start=1;rfam-acc=RF00005;rfam-id=tRNA;score=53.81

        my @a_       = split(/\t/, $line, 9);
		my ($new_id) = $line=~ /rfam-acc=([^;]+)\;/; # RF00005
		$new_id      = "$a_[0]-$new_id.$cpt";        # Rm2011G.c-RF00005.1

		$a_[8]       =~ s/;id=[^;]+;/;/;             # remove 'id=tRNA.1'
		$a_[2]       = "ncRNA"; 

		if ( $o_param->IsDefined('add_parent') )
		{
			# Print the parent line
			$a_[2] = $o_param->Get('add_parent'); 
			for (my $i=0; $i< 8; $i++)
			{
				print $f_out "$a_[$i]\t";
			}
			print $f_out "ID=P_$new_id;Name=$new_id;Ontology_term=SO:0000655;$a_[8]\n";

			# Print the child line
			$a_[2] = "ncRNA";
			for (my $j=0; $j< 8; $j++)
			{
				print $f_out "$a_[$j]\t";
			}
			print $f_out "ID=$new_id;Name=$new_id;Parent=P_$new_id;Ontology_term=SO:0000655;$a_[8]\n";
		}
		else
		{
			for (my $i=0; $i< 8; $i++)
			{
				print $f_out "$a_[$i]\t";
			}
			print $f_out "ID=$new_id;Name=$new_id;Ontology_term=SO:0000655;$a_[8]\n";
		}
		$cpt++;
	}

	$f_in->close;
	$f_out->close;
}

sub Usage
{
print STDERR<<END
$0 
	--help
	--infile     filename  gff rfam (v1.0.2) output format (gzipped allowed)
	--outfile    filename  gff3 format 
[Optional]
	--add_parent string    If defined, add systematically a parent element from this type. (column 3 in GFF3) Examples: 'gene', 'ncRNA'
END
;
}
