#!/usr/bin/perl
#
# $Id: is_running.pl 186 2007-03-16 09:19:47Z manu $
#

=head1 Copyright Notice

 Copyright I.N.R.A. - C.N.R.S. 

 emmanuel.courcelle@toulouse.inra.fr
 jerome.gouzy@toulouse.inra.fr

 This software is a computer program whose purpose is to help you run 
 your scripts on parallel computers.

 This software is governed by the CeCILL  license under French law and
 abiding by the rules of distribution of free software.  You can  use, 
 modify and/ or redistribute the software under the terms of the CeCILL
 license as circulated by CEA, CNRS and INRIA at the following URL
 http://www.cecill.info" 

 As a counterpart to the access to the source code and  rights to copy,
 modify and redistribute granted by the license, users are provided only
 with a limited warranty  and the software's author,  the holder of the
 economic rights,  and the successive licensors  have only  limited
 liability. 

 In this respect, the user's attention is drawn to the risks associated
 with loading,  using,  modifying and/or developing or reproducing the
 software by the user in light of its specific status of free software,
 that may mean  that it is complicated to manipulate,  and  that  also
 therefore means  that it is reserved for developers  and  experienced
 professionals having in-depth computer knowledge. Users are therefore
 encouraged to load and test the software's suitability as regards their
 requirements in conditions enabling the security of their systems and/or 
 data to be ensured and,  more generally, to use and operate it in the 
 same conditions as regards security. 

 The fact that you are presently reading this means that you have had
 knowledge of the CeCILL license and that you accept its terms.

=head1 NAME

is_running.pl - Are you still running ?

=head1 SYNOPSIS

is_running.pl pid

=head1 DESCRIPTION

Check the process pid, and print:
   -0 if the process is not known (dead) OR if it is a zombie
   -1 in the other cases

In your scripts, you can write the following:
   my $status = `is_running.pl $process_id`;
   chomp $status;
   if ($status==1) ...
   

=head1 SUBROUTINES

=cut

MAIN:
{
    my $found=0;
    
    # Use ProcessTable, if it exists (better for portability)
    my $is_process_table=0;
    my $module = "Proc::ProcessTable";
    eval("use $module");
    if ($@ eq "")
    {
        $is_process_table = 1;
    }
    
    my $pid = $ARGV[0];
    if ($is_process_table==1)
    {
        my $processes = new Proc::ProcessTable();
        my @fields = $processes->fields();
        if (grep (/state/,@fields)+0==0)
        {
            die "ERROR - The field state does not exist on your architecture";
        }
         foreach my $p ( @{$processes->table} )
         {
             #print $p->pid."  ".$p->state."\n";
             next if ($p->pid != $pid);
             if ($p->state ne 'defunct')
             {
                 $found=1;
                 last;
             }
         }
    }
    else
    {
        # WARNING - NOT SURE THIS LINE IS CORRECT ON YOUR SYSTEM
        #           YOU SHOULD ADJUST THE COMMAND TO KNOW IF $pid IS ALIVE, AND IF IT IS NOT A ZOMBIE
        #           BUT THE BEST THING WOULD BE TO INSTALL the Proc::ProcessTable module
        #
        my $ps_cmd = "ps -el | grep -w " . $pid . " | egrep -v '^[0145] *Z' | wc -l";
        my $rvl = `$ps_cmd`;
        chomp($rvl);
        $found=1 if ($rvl !=0);
    }
    
    print "$found\n";
    exit(0);
}


