#!/usr/bin/perl

#
# $Id
#

# Copyright INRA

# Erika.Sallet@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Schiex@toulouse.inra.fr

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod 

=cut


BEGIN
{
	$ENV{LANG} = "C";

	use Cwd 'abs_path';
	use File::Basename;
	my ($dirprg) =  dirname(abs_path($0));
	unshift(@INC,"$dirprg/../../corelib");
}

use strict;
use Carp;
use ParamParser;
use General;

MAIN:
{
    my $o_param = New ParamParser('GETOPTLONG',\&Usage,'in=s','out=s');

    $o_param->AssertFileExists('in');
    $o_param->AssertDefined('out');
	my %h_seqid2len=();
	my $id='';
	my $f_in  = $o_param->GetStreamIn('in');
	while(my $l=<$f_in>)
	{
		chomp($l);
		if ( $l =~ /^>(\S+)/ )
		{
			$id=$1;
		}
		else
		{
			$l=~s/[\d\s]//g;
			$h_seqid2len{$id}+=length($l);
		}
	}
	$f_in->close;
	my $f_out = $o_param->GetStreamOut('out');
	foreach my $id (sort keys(%h_seqid2len))
	{
		print $f_out "$id\t$h_seqid2len{$id}\n";
	}
	$f_out->close;
}


sub Usage
{
    print STDERR<<END
Extract sequence lengths
$0
   --in    filename    multifasta file
   --out   filename    tabulated file: seqid\\tseqlen
END
}
