#!/usr/bin/perl

#
# $Id
#

# Copyright INRA

# Erika.Sallet@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Schiex@toulouse.inra.fr

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod 

=head1 NAME


=head1 SYNOPSIS


=head1 DESCRIPTION


=cut

BEGIN
{
    $ENV{LANG} = "C";

    use Cwd 'abs_path';
    use File::Basename;
    my ($dirprg) = dirname(abs_path($0));
    unshift(@INC, "$dirprg/../../../lib");
    unshift(@INC, "$dirprg/../../../corelib");

}

use strict;
use warnings;
use Carp;
use ParamParser;
use General;
use Cwd;

my $ALLSEQFILE_NAME = "sequences";
my $DIRNAME         = "MyGenomeBrowser";
my $debug = 0;

MAIN:
{
    my $o_param = New ParamParser(
                                  'GETOPTLONG', \&Usage, 'egnpp_workdir=s', 'cfg=s', 'outdir=s', 'clean'
                                  );

    $o_param->SetBehaviour('ignore_space');
    $o_param->Update('ENV', "A");
	$o_param->AssertFileExists('cfg');
    $o_param->AssertFullPath('cfg');
    $o_param->Update($o_param->Get('cfg'), 'A');
    $o_param->SetBehaviour('use_substitution_table');
    $o_param->SetSubstitution('%i', $o_param->Get('EGNPP'));
    $o_param->SetSubstitution('%e', $o_param->Get('EUGENEDIR'));
    $o_param->Update($o_param->Get('cfg'), 'U');
	
	$o_param->AssertDirExists('egnpp_workdir');
	$o_param->AssertDefined('outdir');
	
	$o_param->AssertDefined('evidence_list');
	
	my $outdir        = $o_param->Get('outdir');
	my $egnpp_workdir = $o_param->Get('egnpp_workdir');
	my $sequences     = "$egnpp_workdir/sequences.gz";
	my $seq_fof       = "$egnpp_workdir/entries.fof";
	my $f_            = &General::GetStreamIn($seq_fof);
	my %h_seq;
	while (my $seq = <$f_>)
	{
		chomp($seq);
		my ($path, $name) = $seq =~ /^(.+)\/([^\/]+)$/;
		$h_seq{$name}     = $seq;
	}
	$f_->close;

	mkdir("$outdir") if (!-d "$outdir");
 
	my $f_track = &General::GetStreamOut("$outdir/tracks.bigwig.conf");
	
	my $evidence_list   = $o_param->Get('evidence_list');
	my @a_evidence_list = split(/\s+/, $evidence_list);
   
	foreach my $nb (@a_evidence_list) # '2', '3'
	{
		my $root       = "evidence_" . $nb;
		my $patternkey = $root . "_pat";
		my $formatkey  = $root . "_format"; # fastq
		my $repnamekey = $root. "_name";    # not mandatory
		my $typekey    = $root. "_type";    # orientation 'ope', 'ose'
		
		$o_param->SetUnlessDefined($repnamekey, "replicate$nb");
		
		my $format      = $o_param->Get($formatkey);
		my $repname     = $o_param->Get($repnamekey);
		my $pattern     = $o_param->Get($patternkey);
		my $orientation = $o_param->Get($typekey);
	
		if ($format ne "fastq")
		{
			print STDERR "Warning: only fastq format allowed. Not $format.\n";
			next;
		}
	
		$pattern =~ s/\.?fastq\.gz$//;
		$pattern =~ s/\.?fastq\.xz$//;	
		$pattern .= ".bed.specific.gz";
		
		my $evidencerootname = "$outdir/$nb.$repname";
		if (!-e "$evidencerootname.bw.plus")
		{
			my $list = "";
			foreach my $seq (keys %h_seq)
			{
				mkdir ("$outdir/$seq") if (!-d "$outdir/$seq");
				my $rootname = "$outdir/$seq/$seq.$nb.$repname";
				if (!-e "$rootname.position.gff3")
				{
					# 1  - Concatene les fichiers bed du replicat
					my $repbedfile = "$rootname.bed.specific.gz";
					if (!-e $repbedfile)
					{
						$o_param->Set('pattern_path', "$egnpp_workdir/0001/$seq/*/$seq*$pattern");
						my $ra_files   = $o_param->GetListOfFiles('pattern_path');
            			my $cmd        = "gzip -cd ";
						foreach  my $file (@$ra_files)
		 				{
						$cmd .= " $file ";	
						}
						$cmd .= " | gzip > $repbedfile";
						print "$cmd\n";system($cmd);
					}

					# 2 - On recree les fichiers .positions pour voir tous les reads. 
                	#     EGNPP applique les filtres: scoremax=1 et scalingfactor=1000 ni --min_coverage_percent 95
					my $cde = $o_param->Get('prg_bed_to_expr') . " --scalingfactor 1 --scoremax 10000000000 --mode annotation --min_mean_coverage 0 --bedfile $repbedfile --outprefix $rootname --tag glint";
					$cde .= " --interval --bedpe " if ($orientation =~ /ope/);
					print "$cde\n";	system($cde);
				}
			
				$list .= " $rootname.position.gff3 ";
			}
				
			if (!-e "$evidencerootname.position.gff3")
			{
				my $c = "cat $list > $evidencerootname.position.gff3";
				print "$c\n";system($c);
			}
			
			# 3  - Trasnsformation des fichiers .position en bigwig
			my $cde = $o_param->Get('prg_position2wig') . " --mode jbrowse --position_file $evidencerootname.position.gff3 --wig_file $evidencerootname.wig --sequences $sequences --color \"0,255,0\"";
			print "$cde\n";	system($cde);
		}
			
		# 4 - cleaning
		if ($o_param->IsDefined('clean'))
		{	
			my $cde = "rm $evidencerootname.position.gff3";
			#print "$cde\n"; 
			#system($cde);
		}

		print $f_track "[ tracks.$repname ]\ntype = StrandedPlotPlugin/View/Track/Wiggle/StrandedXYPlot\n";
		print $f_track "storeClass     = StrandedPlotPlugin/Store/SeqFeature/StrandedBigWig\nvariance_band = true\n";
		print $f_track "style.clip_marker_color = orange\nurlTemplate    = bigwig/$nb.$repname.bw\n";
		print $f_track "autoscale = local\nkey  = $repname\ncategory = RNASeq / Stranded\n";
	}
	$f_track->close;
}


sub Usage
{
    print STDERR<<END
$0
[Mandatory]

    --egnpp_workdir  dir         EGNPP work directory
    --cfg           filename    Configuration file
    --outdir


END
}
