#!/usr/bin/perl

#
# $Id
#

# Copyright INRA

# Erika.Sallet@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Schiex@toulouse.inra.fr

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod 

=head1 NAME


=head1 SYNOPSIS



=head1 DESCRIPTION


=cut

BEGIN
{
    $ENV{LANG} = "C";

    use Cwd 'abs_path';
    use File::Basename;
    my ($dirprg) = dirname(abs_path($0));
    unshift(@INC, "$dirprg/../../../lib");
    unshift(@INC, "$dirprg/../../../corelib");

}

use strict;
use warnings;
use Carp;
use ParamParser;
use General;
use Cwd;

my $ALLSEQFILE_NAME = "sequences";
my $DIRNAME         = "MyGenomeBrowser";
my $debug = 0;

MAIN:
{
    my $o_param = New ParamParser(
                                  'GETOPTLONG', \&Usage, 'egnpp_outdir=s', 'egnpp_workingdir=s', 'cfg=s', 'nowig', 'debug'
                                  );

    $o_param->SetBehaviour('ignore_space');
    $o_param->Update('ENV', "A");
	$o_param->AssertFileExists('cfg');
    $o_param->AssertFullPath('cfg');
    $o_param->Update($o_param->Get('cfg'), 'A');
    $o_param->SetBehaviour('use_substitution_table');
    $o_param->SetSubstitution('%i', $o_param->Get('EGNPP'));
    $o_param->SetSubstitution('%e', $o_param->Get('EUGENEDIR'));
    $o_param->Update($o_param->Get('cfg'), 'U');
	
	$o_param->AssertDirExists('egnpp_outdir');
	$o_param->SetUnlessDefined('egnpp_workingdir', $o_param->Get('egnpp_outdir'));
	$o_param->AssertDirExists('egnpp_workingdir');
	$debug = 1 if ($o_param->IsDefined('debug'));

	my $egnppoutdir     = $o_param->Get('egnpp_outdir');
	my $egnppworkingdir = $o_param->Get('egnpp_workingdir');
	
	my $fof      = "$egnppworkingdir/entries.fof";
	$o_param->Set('fof', $fof);
	$o_param->AssertFileExists('fof');

	# Load sequences	
	my @a_seq;
	my $f_fof = $o_param->GetStreamIn('fof');
	while (my $seq = <$f_fof>)
	{
		chomp($seq);
		push(@a_seq, $seq);
	}
	$f_fof->close;
	
	system ("rm $DIRNAME.tar.gz") if (-e "$DIRNAME.tar.gz");
	
    # create a directory for mygenomebrowser
	my $outdir = cwd()."/$DIRNAME";
	mkdir $outdir if ( ! -d $outdir);
	
	my $summary = "$outdir/README";
	my $f_readme = &General::GetStreamOut($summary);
	
	# copy the sequence and the annotation
	foreach my $f ("$egnppoutdir/$ALLSEQFILE_NAME", "$egnppoutdir/$ALLSEQFILE_NAME.gff3")
	{
		system("cp $f $outdir/") if (-e $f);
	}

	# Get the blastX files
	my $list = $o_param->Get('blastx_db_list');    # "2 5 6"
    my @a_db_nb = split(/\s+/, $list);
    foreach my $nb (@a_db_nb)
    {
        my $db = $o_param->Get("blastx_db_$nb\_file");
        $db =~ s/\s+$//;
		my @a_db_files;
		foreach my $seq (@a_seq)
		{
			push(@a_db_files, $seq.".blast$nb.gff3");	
		}
		&CatFiles(\@a_db_files, $outdir."/$ALLSEQFILE_NAME.blast$nb.gff3");
		print $f_readme "$ALLSEQFILE_NAME.blast$nb.gff3\t$db\n";
	}
	
	# Search the rnaseq libraries (position.gff3 files)
	my $seq1 = $a_seq[0];
	my @a_ext;
	foreach my $position_file (`ls $seq1.*.position.gff3`)
	{
		my ($ext) = $position_file =~ /^$seq1.(.+).position.gff3$/;
		push (@a_ext, $ext);
	}

	my @a_position;	
	my $factor = $o_param->Get('prg_bed_to_expr_annot_scalingfactor');
	
	if (!$o_param->IsDefined('nowig'))
	{
	# For each libraries, Get the "position" files read by eugene.
	# ie filtered mapping results (max value=1000, min_coverage_percentage=95)
	# and convert in bigwig
	foreach my $ext (@a_ext)
	{
		@a_position = ();
		print "EXT $ext:\n" if ($debug);
		foreach my $seq (@a_seq)
		{
			my $file = $seq . "." . $ext . ".position.gff3";
			push(@a_position, $file);
		}
		
		&CatFiles(\@a_position, $outdir."/$ALLSEQFILE_NAME.$ext.position.gff3");
		&Pos2BigWig($o_param, "$outdir/$ALLSEQFILE_NAME.$ext.position.gff3", "$outdir/$ALLSEQFILE_NAME.$ext.FILTER", "$egnppoutdir/$ALLSEQFILE_NAME", $factor);
		unlink("$outdir/$ALLSEQFILE_NAME.$ext.position.gff3") if (!$debug);
	}
	
	
	# For each libraries, Get the raw mapping results
	# before the filtering (max_value and min_coverage_percentage) and convert in bigwig	
	foreach my $ext (@a_ext)
	{
		@a_position = ();
		print "EXT $ext:\n" if ($debug);
		foreach my $seq (@a_seq)
		{
			my ($dir, $seqname) = $seq =~ /^(.+)\/([^\/]+)$/;
			my $file = `ls $dir/work*/$seqname.$ext.tstart_position.gff3.gz`;
			chomp($file);
			print "FILE >$file<\n" if ($debug);
			push(@a_position, $file);
		}
		
		&CatFiles(\@a_position, $outdir."/$ALLSEQFILE_NAME.$ext.tstart_position.gff3");
		&Pos2BigWig($o_param, "$outdir/$ALLSEQFILE_NAME.$ext.tstart_position.gff3", "$outdir/$ALLSEQFILE_NAME.$ext.RAW", "$egnppoutdir/$ALLSEQFILE_NAME",  1);
		unlink("$outdir/$ALLSEQFILE_NAME.$ext.tstart_position.gff3") if (!$debug);
	}	


	print $f_readme "Files *RAW* are the results of the RNAseq mapping.\n";
	print $f_readme "Files *FILTER* are the results of the RNAseq mapping, after applying some filters:\n";
	print $f_readme "  1) min_coverage_percent=".$o_param->Get('prg_bed_to_expr_annot_min_coverage_percent')."\n";
	print $f_readme "  2) score_max ceiling=".$o_param->Get('prg_bed_to_expr_annot_scoremax')*$o_param->Get('prg_bed_to_expr_annot_scalingfactor')."\n";
	print $f_readme "  These files are using by EuGene for the annotation.\n";
	}
	
	# Get the ncRNA files
	my @a_ncrna;
	foreach my $seq (@a_seq)
	{
		push(@a_ncrna, $seq.".rfamscan.gff3");
		push(@a_ncrna, $seq.".rnammer.gff3");
		push(@a_ncrna, $seq.".trnascan.gff3");
	}
	&CatFiles(\@a_ncrna, $outdir."/$ALLSEQFILE_NAME.ncRNA.gff3");
	#print $f_readme "$ALLSEQFILE_NAME.ncRNA.gff3\tncRNA\n";	

	# Get the tStart signal
	my @a_tstart;
	foreach my $seq (@a_seq)	
	{
		my ($dir, $seqname) = $seq =~ /^(.+)\/([^\/]+)$/;
		my $file = "$outdir/$seqname.tstart.gff3";
		my $c    = "find $dir -name '*tstart.gff3' -exec grep 'transcript_region' {} \\\; > $file";
		print "$c\n"; # if ($debug);
		system($c);
		push(@a_tstart, $file);
	}
	&CatFiles(\@a_tstart, $outdir."/$ALLSEQFILE_NAME.tstart.gff3");
	foreach my $seq (@a_seq)	
	{
		my ($dir, $seqname) = $seq =~ /^(.+)\/([^\/]+)$/;
		my $file = "$outdir/$seqname.tstart.gff3";
		unlink($file);
	}
	print $f_readme "$ALLSEQFILE_NAME.tstart.gff3\ttranscription_starts\n";	
	$f_readme->close;

	my $c = "tar -cvzf $DIRNAME.tar.gz $DIRNAME/*";
	system($c);
	$c = "rm -rf $DIRNAME";
	system($c) if (!$debug);
}


sub Pos2BigWig
{
	my ($o_param, $position_file, $wig_file, $sequences, $factor) = @_;
	
	
	# Compute bigwig files for this library
	my $cmd = $o_param->Get('EGNPP')."/bin/int/misc/position2wig.pl --factor $factor --position_file $position_file --wig_file $wig_file --sequences $sequences";
	print "$cmd\n" if ($debug);
	system($cmd);

	return;
}


sub CatFiles
{
    my ($ra_file, $new) = @_;
	
    my $f_full = &General::GetStreamOut($new);
    foreach my $file (@$ra_file)
    {
		next if (!-e $file);
        my $f_ = &General::GetStreamIn($file);
        while (my $line = <$f_>)
        {
            print $f_full $line;
        }
        $f_->close;
    }
    $f_full->close;

    return;
}

sub Usage
{
    print STDERR<<END
$0
[Mandatory]

    --egnpp_outdir  dir         EGNPP out directory
	--cfg           filename    Configuration file
[Optional]
    --egnpp_workingdir dir      EGNPP working directory	
	--nowig                     Not included wig files of RNAseq alignments


END
}
