#!/usr/bin/perl
#
# $Id
#

# Copyright INRA

# Erika.Sallet@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Schiex@toulouse.inra.fr

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod 

=head1 NAME


=head1 SYNOPSIS

=head1 DESCRIPTION


=cut

BEGIN
{
	$ENV{LANG} = "C";

	use Cwd 'abs_path';
	use File::Basename;
	my ($dirprg) =  dirname(abs_path($0));
	unshift(@INC,"$dirprg/../../../lib");
	unshift(@INC,"$dirprg/../../../corelib");

}

use strict;
use ParamParser;
use General;
use GeneralBioinfo;


my %h_color = (
	21 => '0,0,255',
	22 => '0,0,139',
	23 => '58,95,205',
	27 => '0,255,0',
	28 => '127,255,0',
	29 => '0,255,127',
	36 => '255,0,0',
	37 => '255,99,71',
	38 => '240,128,128',
	39 => '208,32,144',
	40 => '238,130,238',
	41 => '219,112,147',
	4  => '255,255,0',
	60 => '255,165,0', # orange
	'21-22-23' => '0,0,255',
	'27-28-29' => '0,255,0',
	'36-37-38' => '255,0,0',
	'39-40-41' => '208,32,144',
	'30-31-32' => '255,99,71',
	'33-34-35' => '238,130,238',
	'51-52-53' => '0,255,255',     # cyan
	'ZI.1'       => '255,236,139', # yellow
	'ZI.2'       => '255,236,139',
	'ZI.3'       => '255,236,139',
	'ZI.1-2-3'   => '255,236,139',
	'ZIId.1'     => '255,160,122', # salmon
	'ZIId.2'     => '255,160,122',
	'ZIId.3'     => '255,160,122',	
	'ZIId.1-2-3' => '255,160,122',
	'ZIIp.1'     => '255,28,174',  # rose
	'ZIIp.2'     => '255,28,174',	
	'ZIIp.3'     => '255,28,174',	
	'ZIIp.1-2-3' => '255,28,174',
	'IZ.1'       => '191,62,255',  # violet
	'IZ.2'       => '191,62,255',
	'IZ.3'       => '191,62,255',
	'IZ.1-2-3'   => '191,62,255',
	'ZIII.1'     => '104,34,139', # dark violet
	'ZIII.2'     => '104,34,139',
	'ZIII.3'     => '104,34,139',
	'ZIII.1-2-3' => '104,34,139',
	'ZV.1'       => '35,35,142', #	dark blue
	'ZV.2'       => '35,35,142',	
	'ZV.3'       => '35,35,142',
	'ZV.1-2-3'   => '35,35,142',
	'ZVcont.1'     => '77,77,255',  #	 clear blue
	'ZVcont.2'     => '77,77,255',
	'ZVcont.3'     => '77,77,255',
	'ZVcont.1-2-3' => '77,77,255',
	);


my $DEF_minus_color = '130,130,130'; # gris	
my $DEF_plus_color  = '0,0,255';     # bleu

use constant DEFAULT_PREV_SCALE  => 1;
use constant DEFAULT_THRESHOLD   => 1000000000000000;
use constant DEFAULT_MUL_LOG_VAL => 10;

MAIN:
{
	my $o_param =  New ParamParser;
    $o_param->SetUsage(my $usage = sub {&Usage();});

	$o_param->Update("GETOPTLONG","I", ("sequences=s", "replicon=s", "position_file=s", "wig_file=s", "color_code=s", "factor=i", "threshold=i",
										"log_factor=i", "log", "color=s", 'strand=s', 'negative', 'mode=s')); 
	
	
	$o_param->AssertFileExists('position_file');
	$o_param->AssertDefined('wig_file');	
	$o_param->SetUnlessDefined('factor',      &DEFAULT_PREV_SCALE );	
	$o_param->SetUnlessDefined('log_factor',  &DEFAULT_MUL_LOG_VAL );
	$o_param->SetUnlessDefined('threshold',   &DEFAULT_THRESHOLD );
	$o_param->SetUnlessDefined('mode', "jbrowse");
	$o_param->AssertAllowedValue('mode', 'jbrowse', 'apollo');
 
	my $factor     = $o_param->Get('factor');
	my $log_factor = $o_param->Get('log_factor');	
	my $threshold  = $o_param->Get('threshold');
	
	my $position_file =	$o_param->Get('position_file');	
	my $wig_file      = $o_param->Get('wig_file');	

	# select color for plus and minus strand
	my $plus_color  = $DEF_plus_color;
	my $minus_color = $DEF_minus_color;
	if ($o_param->IsDefined('color'))
	{
		$plus_color  = $o_param->Get('color');
		$minus_color = $o_param->Get('color');
	}
	elsif ($o_param->IsDefined('color_code') && defined $h_color{$o_param->Get('color_code')})
	{
		$plus_color  = $h_color{$o_param->Get('color_code')};
	}
	
	if ($o_param->Get('mode') eq "apollo")
	{
		my $negative = 0;
		$negative = 1 if ($o_param->IsDefined('negative'));
		$o_param->AssertDefined('replicon');
		my $replicon = $o_param->Get('replicon');
		$o_param->AssertAllowedValue('strand', 'minus', 'plus', 'both') if ($o_param->IsDefined('strand'));
		my $wanted_strand = "both";
		$wanted_strand = $o_param->Get('strand') if ($o_param->IsDefined('strand'));

		# un fichier avec deux pistes, une par brin avec des couleurs différentes
		# un seul replicon
		my @a_wig_forward;
		my @a_wig_reverse;
		
		my $max1_val = 0;
		my $max2_val = 0;
		$max1_val = &PrintStrand($position_file, \@a_wig_forward, $o_param, "+", $factor, $threshold, $log_factor) if ($wanted_strand eq "both" || $wanted_strand eq "plus");
		$max2_val = &PrintStrand($position_file, \@a_wig_reverse, $o_param, "-", $factor, $threshold, $log_factor) if ($wanted_strand eq "both" || $wanted_strand eq "minus");
		my $max   = General::Max($max1_val, $max2_val);
			
		&PrintWig($wig_file, \@a_wig_forward, \@a_wig_reverse, $replicon, $max, $plus_color, $minus_color, $wanted_strand, $negative);
	}
	else
	{
		$o_param->AssertFileExists('sequences');
	 
		#&Print($position_file, $o_param,$factor, $threshold, $log_factor);
		$wig_file =~ s/\.wig//;
		
		my $replicon = "";
		my $plus_wig  = $wig_file. ".plus.wig";
		my $minus_wig = $wig_file. ".minus.wig";
		my $f_plus  = &General::GetStreamOut($plus_wig);
		my $f_minus = &General::GetStreamOut($minus_wig);
		print $f_plus  "track type=wiggle_0 color=$plus_color\n";
		print $f_minus "track type=wiggle_0 color=$minus_color\n";
		
		my $f_pos = &General::GetStreamIn($position_file);
		while (my $line = <$f_pos>)
		{
			next if $line =~ /^#/;
			my @a_ = split(/\t/, $line);
			my $chr    = $a_[0];
			my $pos    = $a_[3];
			my $val    = $a_[5] * $factor;
			my $strand = $a_[6];
			
			if ($o_param->IsDefined('log'))
			{
				$val = sprintf ("%0.3f", log($val)/log(2));
				$val = $val*$log_factor; 
			}
			else
			{
				$val = $threshold if ($val > $threshold);
			}
			$val = "-".$val if ($strand eq '-');

			if ($chr ne $replicon)
			{
				print $f_plus  "variableStep chrom=$chr\n";
				print $f_minus "variableStep chrom=$chr\n";
				$replicon = $chr;
			}
			if ($strand eq "+")
			{
				print $f_plus "$pos\t$val\n";
			}
			else
			{
				print $f_minus "$pos\t$val\n";
			}			
		}
		$f_pos->close();

		$f_plus->close;
		$f_minus->close;

		if (&GetExePath("wigToBigWig"))
		{
			# convert to bigwig
			#1 create the chrom.sizes file
			my $chrsizes = $wig_file.".sizes";
			my $plus_bw  = $wig_file. ".bw.plus";
			my $minus_bw = $wig_file. ".bw.minus";
			&CreateChrSizeFile($o_param->Get('sequences'), $chrsizes);
			my $cmd = "wigToBigWig $plus_wig $chrsizes $plus_bw";
			system($cmd);
			my $cmd = "wigToBigWig $minus_wig $chrsizes $minus_bw";
			system($cmd);
			unlink($plus_wig,$minus_wig,$chrsizes);
		}
		else
		{
			print STDERR "wigToBigWig not in the path. Only generates wig files\n";
		}
	}
	

}


sub PrintWig
{
	my ($wig_file, $ra_wig_forward, $ra_wig_reverse, $replicon, $max_val, $plus_color, $minus_color, $strand, $negative) = @_;

	my $f_wig = General::GetStreamOut($wig_file);
	
	if ($strand eq "both" || $strand eq "plus")
	{
		
		# Print forward data
		print $f_wig "track type=wiggle_0 color=".$plus_color."\n";
		print $f_wig "variableStep chrom=$replicon\n";
		print  $f_wig "0\t1\n0\t$max_val\n" if ($strand eq "both");
		foreach my $line (@$ra_wig_forward)
		{
			print $f_wig $line;
		}
	}

	if ($strand eq "both" || $strand eq "minus")
	{
		# Print reverse data
		print $f_wig "track color=".$minus_color."\n";
		print $f_wig "variableStep chrom=$replicon\n";
		if ($negative)
		{
		    print  $f_wig "0\t-1\n0\t-$max_val\n" if ($strand eq "both");
		}
		else
		{
		    print  $f_wig "0\t1\n0\t$max_val\n" if ($strand eq "both");;
		}
		foreach my $line (@$ra_wig_reverse)
		{
			print $f_wig $line;
		}
	}
	
	$f_wig->close();

	return;
		
}

sub PrintStrand
{
	my ($position_file, $ra_wig, $o_param, $wanted_strand, $factor, $threshold, $log_factor) = @_;
	
	my $max_val = 0;
	my $f_pos = &General::GetStreamIn($position_file) ;

	
	while (my $line = <$f_pos>)
	{
		my @a_ = split(/\t/, $line);
		my $pos    = $a_[3];
		my $val    = $a_[5] * $factor;
		my $strand = $a_[6];
		
		if ($o_param->IsDefined('log'))
		{
			$val = sprintf ("%0.3f", log($val)/log(2));
			$val = $val*$log_factor; 
		}
		else
		{
			$val = $threshold if ($val > $threshold);
		}
		$val = "-".$val if ($strand eq '-');
		#print $f_wig "$pos\t$val\n" if ($strand eq $wanted_strand);
		push (@$ra_wig, "$pos\t$val\n") if ($strand eq $wanted_strand);

		$max_val = abs($val) if (abs($val)> $max_val);
	}
	$f_pos->close();

	return $max_val;
}

sub CreateChrSizeFile
{
	my ($seq, $file) = @_;

	my ($seqpath, $seqname) = $seq =~ /(.+)\/(.+)/; 

	# Get name and length of the sequence
	my %h_seq;
	&FastaToHash($seq, \%h_seq, 1);

	my $f_out = &General::GetStreamOut($file);
	
	#my @a_keys = keys(%h_seq);
	foreach my $seqid (keys %h_seq)
	{
		my $seqlen = $h_seq{$seqid}{len};
		print $f_out "$seqid $seqlen\n";
	}
	$f_out->close;

	return;
}

=item Usage

	$o_param->Usage();
	Print the usage of the program

=cut

sub Usage
{
    print STDERR<<END
$0     
Convert position file into wig file(s)

[Mandatory]
  --mode           apollo|jbrowse [jbrowse] Build wig file loadable in Apollo or JBrowse genome browser
  --position_file  gff3 file      egnpp GFF3 position file
  --wig_file                      wig output file
  --sequences       fasta          Multifasta of the sequences [Mandatory in jbrowse mode]
  --replicon        string        Replicon name                [Mandatory in apollo mode]

[Optional for both modes]
  --threshold   integer           [Default +inf]   Example: 300  
  --factor      integer           [Default 1]  Multiply each value with this factor. 
                                  Can be useful if the value was previously divided. Example: 1000
  --log                           Convert in log
  --log_factor  integer           [Default 10] If log, multiply each value with this factor.
  --color       string            rgb color code. Example: '0,0,255'
  --color_code  string            To use preselected colors. Example: '20-21-22'

[Optional. Only in apollo mode]
  --strand     minus|plus|both    [Defaut both]
  --negative   true|false         [default true]To create negative value for features in minus strand
  --help
END
      ;
}
