#!/usr/bin/perl

# $Id$

=pod
=head1 NAME
 

=head1  SYNOPSIS


Build file tree, prepare configuration file and launch EuGene-PP

=head1 DESCRIPTION

egnpp_wrapper.pl --format_1 fastq --evidence_files_1 file1.fastq.gz,file2.fastq.gz  --genome genome.fna --organism meliloti  
  --orientation_1 ope --oriented_end_1 1 --small_1 0 --output genome.gff3 

=cut

BEGIN
{
	$ENV{LANG} = "C";
}

use FindBin;
use lib "$FindBin::Bin/../../corelib";
use lib "$FindBin::Bin/../../lib/"; 


use strict;
use warnings;
use Carp;
use ParamParser;
use General;
use GeneralBioinfo;
use Logger;

my $DEBUG       = 0;
my $DEFAULT_CFG = "$FindBin::Bin/../../cfg/egnpp_wrapper.cfg";

MAIN:
{
    my $o_param = New ParamParser('GETOPTLONG',\&Usage, 'genome=s', 'organism=s', 'output=s', 'statistics=s', 'outgenome=s',
                                  'training_db=s',
								  'format_1=s', 'evidence_files_1=s', 'orientation_1=s', 'oriented_end_1=i', 'small_1=i',
                                  'format_2=s', 'evidence_files_2=s', 'orientation_2=s', 'oriented_end_2=i', 'small_2=i',
                                  'format_3=s', 'evidence_files_3=s', 'orientation_3=s', 'oriented_end_3=i', 'small_3=i',
                                  'format_4=s', 'evidence_files_4=s', 'orientation_4=s', 'oriented_end_4=i', 'small_4=i',
                                  'format_5=s', 'evidence_files_5=s', 'orientation_5=s', 'oriented_end_5=i', 'small_5=i',
                                  'blastx_db_1=s', 'blastx_weight_1=s',
								  'blastx_db_2=s', 'blastx_weight_2=s',
								  'blastx_db_3=s', 'blastx_weight_3=s',
								  'blastx_db_4=s', 'blastx_weight_4=s',
								  'blastx_db_5=s', 'blastx_weight_5=s',
								  'no_ncrna_detection', 'cfg=s', 'noclean'
		);

    $o_param->SetUnlessDefined('cfg', $DEFAULT_CFG);	
	$o_param->AssertFileExists('cfg');
    $o_param->Update($o_param->Get('cfg'),'A');

    # Parameters read in the cfg file
    $o_param->AssertFileExists('default_training_db', 'default_blastx_db_1');
    $o_param->AssertDefined('default_blastx_weight_1');


    $o_param->SetUnlessDefined('training_db', $o_param->Get('default_training_db'));
    if (!$o_param->IsDefined('blastx_db_1'))
    {   
        $o_param->Set('blastx_db_1', $o_param->Get('default_blastx_db_1'));
        $o_param->Set('blastx_weight_1', $o_param->Get('default_blastx_weight_1'));      
    }
	$o_param->SetUnlessDefined('organism', "no_name");	
	$o_param->AssertFileExists('genome');
	$o_param->AssertDefined('organism', 'output', 'statistics');
    $o_param->AssertDirExists('EUGENEDIR', 'EGNPPDIR');
    $o_param->AssertFileExists('TEMPLATE_CFG_FILE');

	my @a_index;
	for (my $i=1; $i <= 5 ; $i++)
	{
		next if (!$o_param->IsDefined("evidence_files_$i"));		
		push(@a_index, $i);
		$o_param->SetUnlessDefined("small_$i", "0");
		$o_param->AssertAllowedValue("format_$i", 'fasta', 'fastq');
		$o_param->AssertAllowedValue("orientation_$i", 'ope', 'ose');
		$o_param->AssertAllowedValue("oriented_end_$i", '1', '2');
		$o_param->AssertAllowedValue("small_$i", '0', '1');
	}

	my @a_blastx_ind;
	for (my $j=1; $j <= 5 ; $j++)
	{
		next if (!$o_param->IsDefined("blastx_db_$j"));
		$o_param->AssertFileExists("blastx_db_$j");
		$o_param->SetUnlessDefined("blastx_weight_$j", "0.05");
		push(@a_blastx_ind, $j);
	}
	
	
	if ($o_param->IsDefined('training_db'))
	{
		$o_param->AssertFileExists ('training_db');
		$o_param->AssertFullPath ('training_db');
	}
	
	
	my $rootdir   = "/tmp/egnpp.$$";
	my $inputdir  = "$rootdir/egnpp_input";
	my $outputdir = "$rootdir/egnpp_output";

	&CreateWorkArbo($o_param, $inputdir, $outputdir, \@a_index);
	print "Root dir : $rootdir \n" if ($DEBUG);
	
	&CopyAndFormatDb($o_param, $rootdir, 'training_db') if ($o_param->IsDefined('training_db'));	
	foreach my $i (@a_blastx_ind)
	{
		&CopyAndFormatDb($o_param, $rootdir, "blastx_db_${i}");
	}

	my $cfg_file = "$rootdir/egnpp.cfg";
	&CreateCfgFile($o_param, $o_param->Get('TEMPLATE_CFG_FILE'), $cfg_file, \@a_index, \@a_blastx_ind);
	

	# Launch EuGenePP
    my $EGNPP = $o_param->Get('EGNPPDIR');
	$ENV{EGNPP}     = $EGNPP;
	$ENV{EUGENEDIR} = $o_param->Get('EUGENEDIR');
	my $cmd = "$EGNPP/bin/int/egn-prok.pl --indir $inputdir --outdir  $outputdir --cfg $cfg_file";	
	$cmd .= " --no_ncrna_detection " if ($o_param->IsDefined('no_ncrna_detection'));

	print "$cmd\n" if ($DEBUG);
	system($cmd);


	# copy sequences.gff3
	my $output = $o_param->Get('output');
	$cmd = "cp $outputdir/sequences.gff3 $output";
	print "$cmd\n" if ($DEBUG);
	system($cmd);
	
	#SC:20140415 copy general statistics file
	my $statistics = $o_param->Get('statistics');
	$cmd = "cp $outputdir/sequences.general_statistics.xls $statistics";
	print "$cmd\n" if ($DEBUG);
	system($cmd);
	
	my $outgenome = $o_param->Get('outgenome');
	$cmd = "cp $outputdir/sequences $outgenome";
	print "$cmd\n" if ($DEBUG);
	system($cmd);
    
    # remove working dir
	$cmd = "rm -rf  $rootdir" if (!$o_param->IsDefined('noclean'));
   system($cmd); 
}




sub CopyAndFormatDb
{
	my ($o_param, $rootdir, $file_name) = @_;
	
	my $file         = $o_param->Get($file_name);
	my ($dir, $name) = $file =~ /(.+)\/([^\/]+)$/;
	my $new_file     = "$rootdir/$name";
	
	my $cmd = "cp $file $new_file";
	system($cmd);
	
	# makeblastdb
	$cmd = $o_param->Get('EGNPPDIR')."/bin/ext/ncbi-blast/bin/makeblastdb -in $new_file -dbtype prot -parse_seqids";
	system($cmd);
	
	$o_param->Set($file_name, $new_file);
	
	return;
}


# Creation of temporary directory and subdirectories
# Copy input files in the appropritaed directory
sub CreateWorkArbo
{
	my ($o_param, $inputdir, $outputdir, $ra_index) = @_;
	
	my $datadir   = "$inputdir/data";
	my $genomedir = "$inputdir/genome";
	my $cmd;

	# Creation of temporary directory and subdirectories
	$cmd = "mkdir -p $inputdir";
	system($cmd);
	$cmd = "mkdir -p $outputdir";
	system($cmd); 
	$cmd = "mkdir $datadir; mkdir $genomedir";
	system($cmd);
	
	# Copy fasta genome file in the appropriated directory
	$cmd = "cp " . $o_param->Get('genome') . " $genomedir/";
	system($cmd);
	
	# copy evidence files in the datadir 
	foreach my $i (@$ra_index)
	{
		my $ra_readfiles = $o_param->GetListOfFiles("evidence_files_$i");
        my $cpt_file     = 1; # need to get a number to the files to keep them sorted.
		foreach my $evfile (@$ra_readfiles) 
		{
            # A correct extension is mandatory to allow egnpp to read evidence files
            my $extension = "";
            $extension = ".gz" if (-B $evfile && $evfile !~ /\.gz$/);
			my ($evdir, $evname) = $evfile =~ /(.+)\/([^\/]+)$/;
			$cmd = "cp  $evfile $datadir/__${i}__file${cpt_file}__$evname$extension";
			system($cmd);
            print $cmd if ($DEBUG);
            $cpt_file++;
		}
	}
	
	#system($cmd);

	return;
}

# Create a configuration file according to parameter values 
# based on a template cfg file
sub CreateCfgFile
{
	my ($o_param, $cfg_template, $cfg_new, $ra_index, $ra_blastx) = @_;
	
	my $organism    = $o_param->Get('organism');
	my $training_db = "";
	$training_db    = $o_param->Get('training_db') if ($o_param->IsDefined('training_db'));

	my $format;
	my $orientation;  
	my $oriented_end;
	my $small;
	my $db;
	my $weight;
	my $f_template = &General::GetStreamIn($cfg_template);
	my $f_new      = &General::GetStreamOut($cfg_new);
	while (my $line = <$f_template>)
	{
		$line =~ s/^organism=/organism=$organism/;
		$line =~ s/^training_db_file=/training_db_file=$training_db/ if ($training_db ne "");
		$line = "evidence_list=" . join(" ", @$ra_index)   if ($line =~ /^evidence_list=/);
		$line = "blastx_db_list=" . join(" ", @$ra_blastx) if ($line =~ /^blastx_db_list=/ && scalar(@$ra_blastx) > 0);

		foreach my $i (@$ra_index)
		{
			$format       = $o_param->Get("format_$i");
			$orientation  = $o_param->Get("orientation_$i");
			$oriented_end = $o_param->Get("oriented_end_$i");
			$small        = $o_param->Get("small_$i");
			$line =~ s/evidence_${i}_format=/evidence_${i}_format=$format/;
			$line =~ s/evidence_${i}_type=/evidence_${i}_type=$orientation/;
			$line =~ s/evidence_${i}_oriented_end=/evidence_${i}_oriented_end=$oriented_end/;
			$line =~ s/evidence_${i}_small=/evidence_${i}_small=$small/;
			$line =~ s/evidence_${i}_pat=/evidence_${i}_pat=__${i}__\*/;
		}
		
		foreach my $j (@$ra_blastx)
		{
			$db     = $o_param->Get("blastx_db_${j}");
			$weight = $o_param->Get("blastx_weight_${j}");
			$line =~ s/blastx_db_${j}_file=.*$/blastx_db_${j}_file=$db/;
			$line =~ s/blastx_db_${j}_weight=.*$/blastx_db_${j}_weight=$weight/;
		}
		
		print $f_new $line;
	}
	
	$f_template->close;
	$f_new->close;
	
	
	return;
}


sub Usage
{
    print STDERR<<END
$0
[Mandatory]

   --genome            file     Fasta/multifasta file of the genomic sequence to annotate
   --output            file     GFF3 output file
   --statistics        file     Text file. Basic statistics on the annotation 
   --outgenome         file     Fasta/multifasta output file of the genomic sequences, 
                                order by length. Sequence names could be reformated
[Optional]
   --organism          string 
   --training_db       file     Fasta file. Protein database for eugene training
   --no_ncrna_detection flag      
   --cfg               file     Default EGNPP_WRAPPER_ROOT_DIR/cfg/egnpp_wrapper.cfg.
  --noclean
[Optional, for X = 1..5]
   --format_X          string   [fasta,fastq]
   --evidence_files_X  string   List of fasta or fastq files, separated by a comma
                                (gzipped files or uncompressed files)
   --orientation_X     string   [ose, ope] Oriented single end or oriented paired end 
   --oriented_end_X    integer  [1 or 2]  specification of the insert end which gives the orientation 
                                of the transcript (depends on protocols)
   --small_X           integer  [0 or 1] Default 0. 1 if small inserts analysis.

[Optional for Y = 1..5]
   --blastx_db_Y       Fasta file  BlastX protein database Y.
   --blastx_weight_Y   Float       Default 0.05. Weight of the database Y


END
}


