#
# $Id: Embl.pm 1288 2013-10-21 08:39:57Z manu $
#
 
package AnnotatedSeqFileReader::Embl;

=head1 NAME

AnnotatedSeqFileReader::Embl - parse an Embl file and load an AnnotatedSeq object, derives from UsingBioPerl
                               See http://www.ncbi.nlm.nih.gov/Sitemap/samplerecord.html

=head1 SYNOPSIS
	See AnnotatedSeq

=head1 DESCRIPTION

=cut

use strict;
use warnings;
use Carp;

use Error qw(:try);
use Bio::SeqIO;
use Bio::Seq;
use General;
use AnnotatedSeq;
use Data::Dumper;

use base qw(AnnotatedSeqFileReader::UsingBioPerl);

BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 1288 $ =~ /\d+/g); $r[0]};
}

=head2 procedure _Init

 Title        : _Init
 Usage        : my $o_gbk = New AnnotatedSeqFile::Embl(...);
 Function     : Constructor

=cut

sub _Init
{
	my $self = shift;
	push @_, 'embl';
	$self->SUPER::_Init(@_);
}

=head2 procedure DetectContig

 Title        : DetectContig
 Usage        : my $ctg = $o_gbk->DetectContig();
 Function     : Open the gbk file and read it until a tabulated LOCUS line is found, and read the contig/chromosome label

=cut
sub DetectContig
{
	my $self    = shift;
	my $o_annot = $self -> { _o_annot };
	my $f_embl = &GetStreamIn($o_annot->{__in_ann_file});
	my $ctg = "";
	while( my $line = <$f_embl>)
	{
		chomp($line);
		if ( $line =~ /^ID   ([^;]+);/ )
		{
			$ctg = $1;
			last;
		}
		else
		{
			next;
		}
	}
	if ($ctg =~ /\s/)
	{
		throw LipmError ( -text => "Your Embl file is NOT valid: ID = \"$ctg\"" );
	}
	$o_annot -> { __contig } = $ctg;
}

=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license - www.cecill.info

=cut

1;
