package AnnotatedSeqFileWriter::Tbl;

=head1 NAME

AnnotatedSeqFileWriter::Tbl - Dump an AnnotatedSeq object to a tbl file, derives from UsingBioPerl
							  As there is no driver in bioperl for the tbl data format, we overwrite the _NewSeqIO function,
							  which returns a AnnotatedSeqFileWriter::TblSeqIO object
                              See http://www.ncbi.nlm.nih.gov/Sitemap/samplerecord.html

=head1 SYNOPSIS
	See AnnotatedSeq

=head1 DESCRIPTION

=cut
use strict;
use warnings;
use Carp;

use Error qw(:try);
use Bio::Seq;
use Data::Dumper;
use General;

use Locus;
use base qw( AnnotatedSeqFileWriter::UsingBioPerl );

use AnnotatedSeqFileWriter::TblSeqIO;

BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 1009 $ =~ /\d+/g); $r[0]};
}

=head2 procedure _Init

 Title        : _Init
 Usage        : my $o_gbk = New AnnotatedSeqFile::Tbl(...);
 Function     : Constructor, calls the parent's constructor and sets the _for_submission flag to &TRUE

=cut

sub _Init
{
	my $self = shift;
	push @_, 'tbl';
	$self->SUPER::_Init(@_);
}

sub _NewSeqIO
{
	my $self = shift;
	my ($format,$fh_out) = @_;
	return New AnnotatedSeqFileWriter::TblSeqIO($fh_out);
}

=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license - www.cecill.info

=cut

1;


