#
# $Id$
#

package AnnotatedSeqFileWriter::TblSeqIO;

=head1 NAME

AnnotatedSeqFileWriter::Genbank - Dump an AnnotatedSeq object to a Genbank file, derives from UsingBioPerl
                                  See http://www.ncbi.nlm.nih.gov/Sitemap/samplerecord.html

=head1 SYNOPSIS
	See AnnotatedSeq

=head1 DESCRIPTION

=cut
use strict;
use warnings;
use Carp;

use Error qw(:try);
use Bio::Seq;
use Data::Dumper;
use General;

use Locus;
use base qw( LipmObject );

BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 1009 $ =~ /\d+/g); $r[0]};
}

=head2 procedure _Init

 Title        : _Init
 Usage        : my $o_gbk = New AnnotatedSeqFile::Embl(...);
 Function     : Constructor

=cut

sub _Init
{
	my $self = shift;
	my ( $fh_out ) = @_;
	$self->SUPER::_Init();
	$self -> { __fh_out } = $fh_out;
}

=head2 procedure write_seq

Title        : Load
Usage        : $o_gbk->Load();
Prerequisite : __in_ann_file should be an embl/gbk/... file, with only ONE LOCUS, and the file should contain unique "locus_tag"
Function     : Load the protein-coding genes, the ncRNA and the repeat contained from a file, using the BioPerl API
Returns      : none
Args         : none

=cut

sub write_seq
{
	my $self = shift;
	my ($seq) = @_;
	
	my $fh_out = $self -> { __fh_out };
	my $accession_number = $seq->accession_number;
	print $fh_out ">Features $accession_number\n";
	foreach my $f ( $seq->all_SeqFeatures() )
	{		
		my $feature = $f->primary_tag();
		next if ($feature eq 'source' );
		
		my $location = $f->location;
		my $start    = $location->start;
		my $end      = $location->end;
		my $strand   = $f->strand;
		
		# NE MARCHE POUR L'INSTANT QUE POUR LES BACTERIES (une seule location !)
		if ($location->isa('Bio::Location::Split')) 
		{
			my @a_locations = $location->each_Location();
			my $before = ($strand eq '1') ? '<' : '>';
			my $after  = ($strand eq '1') ? '>' : '<';
			foreach my $o_loc (@a_locations) 
			{
				if ($o_loc->isa('Bio::Location::Fuzzy'))
				{
					$start = "$before$start" if ( $o_loc->start_pos_type eq 'BEFORE' );
					$start = "$after$start" if ( $o_loc->start_pos_type eq 'AFTER' );
					$end   = "$before$end" if ( $o_loc->end_pos_type eq 'BEFORE' );
					$end   = "$after$end" if ( $o_loc->end_pos_type eq 'AFTER' );
				}
			}
		}
		print $fh_out ($strand eq '1') ? "$start\t$end\t$feature\n" : "$end\t$start\t$feature\n";
		
		foreach my $qualifier ( $f->all_tags() )
		{
			# do NOT dump the translation qualifier
			next if ( $qualifier eq 'translation' );
			
			foreach my $v ( $f->each_tag_value($qualifier) )
			{
				if ( $qualifier eq 'note' && $v=~',' )
				{
					my @a_v = split ',',$v;
					foreach my $v (@a_v)
					{
						print $fh_out "\t\t\t$qualifier\t$v\n";
					}
				}
				else
				{
					print $fh_out "\t\t\t$qualifier\t$v\n";
				}
			}
		}
	}
}

=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license - www.cecill.info

=cut

1;


