#!/usr/bin/perl

#
# $Id
#

# Copyright INRA

# Erika.Sallet@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Schiex@toulouse.inra.fr

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod 

=cut


BEGIN
{
	$ENV{LANG} = "C";

	use Cwd 'abs_path';
	use File::Basename;
	my ($dirprg) =  dirname(abs_path($0));
	unshift(@INC,"$dirprg/../../../corelib");
}

use strict;
use Carp;
use ParamParser;
use General;
use GeneralBioinfo;
use Data::Dumper;

MAIN:
{
    my $o_param = New ParamParser('GETOPTLONG',\&Usage,'in=s','seqlen=s','scov=i','backup');

#    $o_param->Set('backup',1); # pour debug
	my $ext_bk = time.".$$.filt";

    $o_param->AssertFileExists('seqlen');
    $o_param->AssertDefined('in');
	exit if ( -e $o_param->Get('in') && -z $o_param->Get('in') ); # rien a filtrer 
    $o_param->AssertFileExists('in');
    $o_param->AssertInteger('scov');

	my %h_seqlen=();
	my @a_key=(1);
	my @a_val=(2);
	&TabToHash($o_param->Get('seqlen'),\%h_seqlen,\@a_key,\@a_val);
	foreach my $id (keys(%h_seqlen))
	{
		if  ($id=~/=|;/) # idem lipm_m8_to_gff3.pl
		{
			my $seqid = $id;
			$seqid =~ s/=|;/_/g;
			$h_seqlen{$seqid}=$h_seqlen{$id};
		}
	}

	my $scov = $o_param->Get('scov');
	my $f_in   = $o_param->GetStreamIn('in');
	my $outfile = $o_param->Get('in').".$ext_bk";
	my $f_out  = &GetStreamOut($outfile);
	my %h_res;
	my @a_hit=();
	my $seqid;
	my $match;
	while(my $l=<$f_in>)
	{
		chomp($l);
		&ParseGFF3Line(\$l,2,\%h_res);
		if ( $h_res{type} eq 'match' )
		{
			if ( ! defined($h_seqlen{$seqid}) && scalar(@a_hit) )
			{
				Carp::croak("Error: unable to find the length, check consistency between --seqlen and --in parameters ($seqid)\n");
			}
			&Filter($f_out,$match,\@a_hit,$h_seqlen{$seqid},$scov);
			@a_hit=();
			$match=$l;
			$seqid=$h_res{Name};
		}
		else
		{
			push(@a_hit,$l);
		}
	}
	&Filter($f_out,$match,\@a_hit,$h_seqlen{$seqid},$scov);
	$f_out->close;

	if ( $o_param->IsDefined('backup') )
	{
		system("cp ".$o_param->Get('in'). " $outfile.ori");
	}
	system("mv $outfile ".$o_param->Get('in'));
}

sub Filter
{
my($f_out,$match,$ra_hit,$seqlen,$scov)=@_;

	my %h_res=();
	my $min = 99999999;
	my $max = 0;
	if ( scalar(@$ra_hit) && $seqlen )
	{
		for(my $i=0;$i<scalar(@$ra_hit);$i++)
		{
			&ParseGFF3Line(\$$ra_hit[$i],2,\%h_res);
			my ($id,$start,$end) = split(' ',$h_res{Target});
			$min=&Min($min,$start);
			$max=&Max($max,$end);
		}
		if ( (($max-$min+1)*100/$seqlen) >= $scov )
		{
			print $f_out "$match\n";
			for(my $i=0;$i<scalar(@$ra_hit);$i++)
			{
				print $f_out "$$ra_hit[$i]\n";
			}
		}
	}
}



sub Usage
{
    print STDERR<<END
Extract sequence lengths
$0
Mandatory
   --in     filename    gff3 formatted file (match/match_part)
   --seqlen filename    tabulated file (seqid\\tseqlen);
   --scov   integer     minimum percent span of the target sequence
Optional
   --backup           keep the unfiltered version
END
}
