#
# $Id: Fasta.pm 1342 2013-12-17 12:52:36Z manu $
#

package AnnotatedSeqFileWriter::Fasta;

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


=head1 NAME

AnnotatedSeqFileWriter::Fasta - Using annotations stored in a companion AnnotatedSeq object, using the genomic fasta file
                                dump fasta files corresponding to the features

=head1 SYNOPSIS

 my $o_dumper = New AnnotatedSeqFileWriter::Narcisse( o_annot => $o_annot );
 $o_dumper -> Dump();

=head1 DESCRIPTION

 this class extends the AnnotatedSeqFileWriter, it dumps data from AnnotatedSeqFile to Narcisse files

=cut

use strict;
use warnings;
use Carp;

use Error qw(:try);
use LipmError::IOException;
use Bio::Seq;
use Data::Dumper;
use General;

use Locus;
use base qw( AnnotatedSeqFileWriter::FastaLike );

BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 1342 $ =~ /\d+/g); $r[0]};
}

sub _Init
{
	my $self = shift;

	$self->SUPER::_Init(@_);
}

sub GetFormat
{
	return 'FASTA';
}

=head2 Procedure __PrintFasta

 Title        : __PrintFasta
 Procedure    : Print a sequence fasta-formatted
 Usage        : $self -> __PrinFasta ( $fh_out, $header, \$seq );
 Prerequisite : $fh_out must be an output-opened file handle
 Args         : $fh_out : A file handle
                $header : The fasta header
                $r_seq  : A ref-to-scalar, the sequence to print
                          IT WILL BE MODIFIED !
 Globals      : none

=cut
	
sub _PrintFasta
{
	my $self = shift;
	my ($fh_out,$header,$r_seq) = @_;
	
	my $eol = (length($$r_seq)%60==0) ? "" : "\n";
	$$r_seq  =~ s/(\w{60})/$1\n/g;
	
	#print $fh_out $header . "\n" . uc($$r_seq) . $eol;
	print $fh_out $header . "\n" . $$r_seq . $eol;

}	
=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license - www.cecill.info

=cut

1;

